import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';

///[AiConversationStarterConfiguration] is a data class that has configuration properties
///to customize the functionality and appearance of [AiConversationStarter]
///
/// ```dart
///   AiConversationStarterConfiguration(
///    style: AiConversationStarterStyle(),
///    theme: CometChatTheme(palette: Palette(),typography: Typography())
///  );
/// ```
class AiConversationStarterConfiguration {
  AiConversationStarterConfiguration({
    this.conversationStarterStyle,
    this.theme,
    this.loadingStateText,
    this.errorStateText,
    this.emptyStateText,
    this.conversationStarterView,
    this.onLoad,
    this.loadingIconUrl,
    this.loadingStateView,
    this.errorIconUrl,
    this.errorStateView,
    this.emptyStateView,
    this.emptyIconUrl,
  });

  ///[conversationStarterStyle] provides styling to the reply view
  final AiConversationStarterStyle? conversationStarterStyle;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[emptyStateText] text to be displayed when the replies are empty
  final String? emptyStateText;

  ///[loadingStateText] text to be displayed when loading occur
  final String? loadingStateText;

  ///[errorStateText] text to be displayed when error occur
  final String? errorStateText;

  ///[conversationStarterView] gives conversation starter view
  final Widget Function(List<String> replies, BuildContext context)?
      conversationStarterView;


  ///[onLoad] function when user wants to send custom ui
  final Function(
      User? user,
      Group? group,
      Function(Widget, bool isLoading) onSuccess,
      Function(CometChatException e, bool isError) onError)? onLoad;

  ///[emptyStateView] returns view for empty state
  final WidgetBuilder? emptyStateView;

  ///[loadingStateView] returns view for loading state
  final WidgetBuilder? loadingStateView;

  ///[errorStateView] returns view for error state
  final WidgetBuilder? errorStateView;

  ///[errorIconUrl] used to set the error icon
  final String? errorIconUrl;

  ///[emptyIconUrl] used to set the empty icon
  final String? emptyIconUrl;

  ///[loadingIconUrl] used to set the loading icon
  final String? loadingIconUrl;

  AiConversationStarterConfiguration fromJson(Map<String, dynamic> json) {
    return AiConversationStarterConfiguration(
      conversationStarterStyle: conversationStarterStyle ??
          json["conversationStarterStyle"] ??
          AiConversationStarterStyle().fromJson(json),
      theme: theme ?? json["theme"],
      emptyStateText: emptyStateText ?? json["emptyStateText"],
      loadingStateText: loadingStateText ?? json["loadingStateText"],
      errorStateText: errorStateText ?? json["errorStateText"],
      conversationStarterView:
          conversationStarterView ?? json["conversationStarterView"],
      onLoad: onLoad ?? json["onLoad"],
      emptyStateView: emptyStateView ?? json['emptyStateView'],
      loadingStateView: loadingStateView ?? json['loadingStateView'],
      errorStateView: errorStateView ?? json['errorStateView'],
      errorIconUrl: errorIconUrl ?? json['errorIconUrl'],
      emptyIconUrl: emptyIconUrl ?? json['emptyIconUrl'],
      loadingIconUrl: loadingIconUrl ?? json['loadingIconUrl'],
    );
  }

  /// Copies current [AiConversationStarterConfiguration] with some changes
  AiConversationStarterConfiguration copyWith({
    AiConversationStarterStyle? conversationStarterStyle,
    CometChatTheme? theme,
    String? emptyStateText,
    String? loadingStateText,
    String? errorStateText,
    Widget Function(List<String> replies, BuildContext context)?
        conversationStarterView,
    Widget Function(List<String> replies, BuildContext context)?
        conversationStarterEmptyView,
    Function(
            User? user,
            Group? group,
            Function(Widget, bool isLoading) onSuccess,
            Function(CometChatException e, bool isError) onError)?
        onLoad,
    WidgetBuilder? emptyStateView,
    WidgetBuilder? loadingStateView,
    WidgetBuilder? errorStateView,
    String? errorIconUrl,
    String? emptyIconUrl,
    String? loadingIconUrl,
  }) {
    return AiConversationStarterConfiguration(
      conversationStarterStyle:
          conversationStarterStyle ?? this.conversationStarterStyle,
      theme: theme ?? this.theme,
      emptyStateText: emptyStateText ?? this.emptyStateText,
      loadingStateText: loadingStateText ?? this.loadingStateText,
      errorStateText: errorStateText ?? this.errorStateText,
      conversationStarterView:
          conversationStarterView ?? this.conversationStarterView,
      onLoad: onLoad ?? this.onLoad,
      emptyStateView: emptyStateView ?? this.emptyStateView,
      loadingStateView: loadingStateView ?? this.loadingStateView,
      errorStateView: errorStateView ?? this.errorStateView,
      errorIconUrl: errorIconUrl ?? this.errorIconUrl,
      emptyIconUrl: emptyIconUrl ?? this.emptyIconUrl,
      loadingIconUrl: loadingIconUrl ?? this.loadingIconUrl,
    );
  }

  /// Merges current [AiConversationStarterConfiguration] with [other]
  AiConversationStarterConfiguration merge(
      AiConversationStarterConfiguration? other) {
    if (other == null) return this;
    return copyWith(
      conversationStarterStyle: other.conversationStarterStyle,
      theme: other.theme,
      emptyStateText: other.emptyStateText,
      loadingStateText: other.loadingStateText,
      errorStateText: other.errorStateText,
      conversationStarterView: other.conversationStarterView,
      onLoad: other.onLoad,
      emptyStateView: other.emptyStateView,
      loadingStateView: other.loadingStateView,
      errorStateView: other.errorStateView,
      errorIconUrl: other.errorIconUrl,
      emptyIconUrl: other.emptyIconUrl,
      loadingIconUrl: other.loadingIconUrl,
    );
  }
}
