import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';

///[AiAuxiliaryButton] is the widget that represents the [AiEnabler] in auxiliary button view of the [CometChatMessageComposer]
///
/// ```dart
///
/// AiAuxiliaryButton(
///   buttonIconUrl: AssetConstants.ai,
///   onCloseAiTap: () {
///     // custom action to hide the Ai options
///   },
///   onOpenAiTap: () {
///     // custom action to show the ai button
///   },
///   theme: CometChatTheme.dark(),
/// );
///
/// ```

class AiAuxiliaryButton extends StatefulWidget {
  const AiAuxiliaryButton({
    Key? key,
    this.buttonIconUrl,
    this.buttonIconTint,
    this.onCloseAiTap,
    this.onOpenAiTap,
    this.theme,
  }) : super(key: key);

  ///[buttonIconUrl] shows Ai enabler Button
  final String? buttonIconUrl;

  ///[onOpenAiTap] overrides default action that shows the Ai options
  final VoidCallback? onOpenAiTap;

  ///[onCloseAiTap] overrides default action that hides the Ai options
  final VoidCallback? onCloseAiTap;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[buttonIconTint] provides color to the Ai enabler icon
  final Color? buttonIconTint;

  @override
  State<AiAuxiliaryButton> createState() => _AiAuxiliaryButtonState();
}

class _AiAuxiliaryButtonState extends State<AiAuxiliaryButton>
    with CometChatUIEventListener {
  bool _isAiButtonActive = true;

  late CometChatTheme theme;

  late String dateStamp;
  late String _listenerId;

  @override
  void initState() {
    theme = widget.theme ?? cometChatTheme;
    dateStamp = DateTime.now().microsecondsSinceEpoch.toString();
    _listenerId = "AiAuxiliaryButtonListener";
    CometChatUIEvents.addUiListener(_listenerId, this);
    super.initState();
  }

  @override
  void dispose() {
    CometChatUIEvents.removeUiListener(_listenerId);
    super.dispose();
  }

  @override
  void hidePanel(Map<String, dynamic>? id, CustomUIPosition uiPosition) {
    if (uiPosition == CustomUIPosition.composerBottom &&
        _isAiButtonActive == false) {
      setState(
        () {
          _isAiButtonActive = true;
        },
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return IconButton(
      onPressed: () {
        if (_isAiButtonActive) {
          if (widget.onOpenAiTap != null) {
            widget.onOpenAiTap!();
          }
          setState(() {
            _isAiButtonActive = false;
          });
        } else {
          if (widget.onCloseAiTap != null) {
            widget.onCloseAiTap!();
          }
          setState(() {
            _isAiButtonActive = true;
          });
        }
      },
      icon: Image.asset(
        widget.buttonIconUrl ?? AssetConstants.ai,
        package: UIConstants.packageName,
        color: widget.buttonIconTint ?? theme.palette.getAccent700(),
      ),
    );
  }
}
