import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';

///[AiEnablerConfiguration] is a data class that has configuration properties

class AiEnablerConfiguration {
  AiEnablerConfiguration({
    this.buttonIconUrl,
    this.theme,
    this.buttonIconTint,
    this.buttonTextStyle,
    this.loadingIconUrl,
    this.loadingStateView,
    this.errorIconUrl,
    this.errorStateView,
    this.emptyStateView,
    this.emptyIconUrl,
    this.loadingIconTint,
    this.errorIconTint,
    this.emptyIconTint,
    this.emptyTextStyle,
    this.loadingTextStyle,
    this.errorTextStyle,
  });

  ///[buttonIconUrl] to update ai enabler icon
  final String? buttonIconUrl;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[buttonIconTint] provides color to the Ai Icon Icon/widget
  final Color? buttonIconTint;

  ///[buttonTextStyle] sets TextStyle for button text
  final TextStyle? buttonTextStyle;

  ///[emptyStateView] returns view for empty state
  final WidgetBuilder? emptyStateView;

  ///[loadingStateView] returns view for loading state
  final WidgetBuilder? loadingStateView;

  ///[errorStateView] returns view for error state
  final WidgetBuilder? errorStateView;

  ///[errorIconUrl] used to set the error icon
  final String? errorIconUrl;

  ///[emptyIconUrl] used to set the empty icon
  final String? emptyIconUrl;

  ///[loadingIconUrl] used to set the loading icon
  final String? loadingIconUrl;

  ///[loadingIconTint] provides color to loading icon
  final Color? loadingIconTint;

  ///[emptyTextStyle] provides styling for text to indicate replies list is empty
  final TextStyle? emptyTextStyle;

  ///[errorTextStyle] provides styling for text to indicate some error has occurred while fetching the replies list
  final TextStyle? errorTextStyle;

  ///[errorIconTint] provides color to error icon
  final Color? errorIconTint;

  ///[emptyIconTint] provides color to empty icon
  final Color? emptyIconTint;

  ///[loadingTextStyle] provides styling for text to indicate loading state has occurred while fetching the replies list
  final TextStyle? loadingTextStyle;

  Map<String, dynamic> toJson() => {
        'buttonIconUrl': buttonIconUrl,
        'theme': theme,
        'buttonIconTint': buttonIconTint,
        'buttonTextStyle': buttonTextStyle,
        'loadingIconUrl': loadingIconUrl,
        'loadingStateView': loadingStateView,
        'errorIconUrl': errorIconUrl,
        'errorStateView': errorStateView,
        'emptyStateView': emptyStateView,
        'emptyIconUrl': emptyIconUrl,
        "loadingIconTint": loadingIconTint,
        "errorIconTint": errorIconTint,
        "emptyIconTint": emptyIconTint,
        "emptyTextStyle": emptyTextStyle,
        "loadingTextStyle": loadingTextStyle,
        "errorTextStyle": errorTextStyle,
      };

  /// Copies current [AiEnablerConfiguration] with some changes
  AiEnablerConfiguration copyWith({
    String? buttonIconUrl,
    CometChatTheme? theme,
    Color? buttonIconTint,
    TextStyle? buttonTextStyle,
    WidgetBuilder? emptyStateView,
    WidgetBuilder? loadingStateView,
    WidgetBuilder? errorStateView,
    String? errorIconUrl,
    String? emptyIconUrl,
    String? loadingIconUrl,
    Color? loadingIconTint,
    TextStyle? emptyTextStyle,
    TextStyle? errorTextStyle,
    Color? errorIconTint,
    Color? emptyIconTint,
    TextStyle? loadingTextStyle,
  }) {
    return AiEnablerConfiguration(
      buttonIconUrl: buttonIconUrl ?? this.buttonIconUrl,
      theme: theme ?? this.theme,
      buttonIconTint: buttonIconTint ?? this.buttonIconTint,
      buttonTextStyle: buttonTextStyle ?? this.buttonTextStyle,
      emptyStateView: emptyStateView ?? this.emptyStateView,
      loadingStateView: loadingStateView ?? this.loadingStateView,
      errorStateView: errorStateView ?? this.errorStateView,
      errorIconUrl: errorIconUrl ?? this.errorIconUrl,
      emptyIconUrl: emptyIconUrl ?? this.emptyIconUrl,
      loadingIconUrl: loadingIconUrl ?? this.loadingIconUrl,
      loadingIconTint: loadingIconTint ?? this.loadingIconTint,
      emptyTextStyle: emptyTextStyle ?? this.emptyTextStyle,
      errorTextStyle: errorTextStyle ?? this.errorTextStyle,
      errorIconTint: errorIconTint ?? this.errorIconTint,
      emptyIconTint: emptyIconTint ?? this.emptyIconTint,
      loadingTextStyle: loadingTextStyle ?? this.loadingTextStyle,
    );
  }

  /// Merges current [AiEnablerConfiguration] with [other]
  AiEnablerConfiguration merge(AiEnablerConfiguration? other) {
    if (other == null) return this;
    return copyWith(
      buttonIconUrl: other.buttonIconUrl,
      theme: other.theme,
      buttonIconTint: other.buttonIconTint,
      buttonTextStyle: other.buttonTextStyle,
      emptyStateView: other.emptyStateView,
      loadingStateView: other.loadingStateView,
      errorStateView: other.errorStateView,
      errorIconUrl: other.errorIconUrl,
      emptyIconUrl: other.emptyIconUrl,
      loadingIconUrl: other.loadingIconUrl,
      loadingTextStyle: other.loadingTextStyle,
      emptyIconTint: other.emptyIconTint,
      emptyTextStyle: other.emptyTextStyle,
      errorIconTint: other.errorIconTint,
      errorTextStyle: other.errorTextStyle,
      loadingIconTint: other.loadingIconTint,
    );
  }
}
