import 'dart:io';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart' as cc;

///[AiEnablerDecorator] is a the view model for [AiEnabler] it contains all the relevant business logic
///it is also a sub-class of [DataSourceDecorator] which allows any extension to override the default methods provided by [MessagesDataSource]

class AiEnablerDecorator extends DataSourceDecorator {
  List<AiExtension>? aiFeatureList = [];
  AiEnablerConfiguration? configuration;

  AiEnablerDecorator(DataSource dataSource,
      {this.aiFeatureList,this.configuration})
      : super(dataSource) ;

  @override
  Widget getAuxiliaryOptions(User? user, Group? group, BuildContext context,
      Map<String, dynamic>? id, CometChatTheme? theme) {
    if (aiFeatureList != null && aiFeatureList!.isNotEmpty) {
      List<Widget> auxiliaryButtons = [];

      Widget _widget =
          super.getAuxiliaryOptions(user, group, context, id, theme);

      auxiliaryButtons.add(_widget);

      for (var element in aiFeatureList!) {
        if (element.getExtensionId() ==
            AiFeatureConstants.aiConversationStarters) {
          aiFeatureList?.remove(element);
          break;
        }
      }

      if (id?['parentMessageId'] == null && aiFeatureList!.isNotEmpty) {
        auxiliaryButtons.add(getAiFeatureButton(user, group, context, id));
      }

      return auxiliaryButtons.isEmpty
          ? const SizedBox()
          : Row(
              mainAxisSize: MainAxisSize.min,
              children: [...auxiliaryButtons],
            );
    }
    return const SizedBox();
  }

  getAiFeatureButton(User? user, Group? group, BuildContext context,
      Map<String, dynamic>? id) {
    Map<String, dynamic> _id = {};
    CometChatTheme theme = configuration?.theme ?? cometChatTheme;

    var extensionNames = {
      AiFeatureConstants.aiSmartReplies:
          cc.Translations.of(context).suggest_a_reply,
      AiFeatureConstants.aiConversationStarters:
          cc.Translations.of(context).conversation_starters,
    };
    if (user != null) {
      _id['uid'] = user.uid;
    } else if (group != null) {
      _id['guid'] = group.guid;
    }
    aiButtonTap() async {
      if (Platform.isIOS) {
        List<Widget> featureList = [];
        for (int i = 0; i < aiFeatureList!.length; i++) {
          featureList.add(
            CupertinoActionSheetAction(
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Text(
                  extensionNames[aiFeatureList![i].getExtensionId()] ?? "",
                  style: configuration?.buttonTextStyle ??
                      TextStyle(
                          fontSize: theme.typography.title2.fontSize,
                          fontWeight: theme.typography.title2.fontWeight,
                          color: theme.palette.getAccent(),
                          fontFamily: theme.typography.title2.fontFamily),
                ),
              ),
              onPressed: () {
                Navigator.of(context).pop();
                CometChatUIEvents.onAiFeatureTapped(user, group);
              },
            ),
          );
        }
        showCupertinoModalPopup(
          context: context,
          builder: (_) {
            return CupertinoActionSheet(
              actions: featureList,
            );
          },
        );
      } else if (Platform.isAndroid) {
        showModalBottomSheet(
          context: context,
          isScrollControlled: true,
          isDismissible: true,
          backgroundColor: theme.palette.getBackground(),
          shape: const RoundedRectangleBorder(
            borderRadius: BorderRadius.vertical(
              top: Radius.circular(16),
            ),
          ),
          builder: (builder) {
            return ListView.builder(
              shrinkWrap: true,
              itemCount: aiFeatureList?.length,
              itemBuilder: (_, int index) {
                return Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: GestureDetector(
                    onTap: () {
                      Navigator.of(context).pop();
                      CometChatUIEvents.onAiFeatureTapped(user, group);
                    },
                    child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Text(
                        extensionNames[
                                aiFeatureList![index].getExtensionId()] ??
                            "",
                        style: configuration?.buttonTextStyle ??
                            TextStyle(
                                fontSize: theme.typography.title2.fontSize,
                                fontWeight: theme.typography.title2.fontWeight,
                                color: theme.palette.getAccent(),
                                fontFamily: theme.typography.title2.fontFamily),
                      ),
                    ),
                  ),
                );
              },
            );
          },
        );
      }
    }

    closeAiTap() {
      CometChatUIEvents.hidePanel(
        _id,
        CustomUIPosition.composerBottom,
      );
    }

    if (aiFeatureList == null || aiFeatureList!.isEmpty) {
      return;
    }

    return AiAuxiliaryButton(
      buttonIconUrl: configuration?.buttonIconUrl,
      onCloseAiTap: closeAiTap,
      onOpenAiTap: aiButtonTap,
      theme: configuration?.theme,
      buttonIconTint: configuration?.buttonIconTint,
    );
  }

  @override
  String getId() {
    return "ai_features";
  }
}
