import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';

class AiSmartReplyConfiguration {
  AiSmartReplyConfiguration({
    this.smartRepliesStyle,
    this.theme,
    this.errorStateText,
    this.emptyStateText,
    this.onError,
    this.smartReplyView,
    this.onClick,
    this.loadingStateText,
    this.loadingIconUrl,
    this.loadingStateView,
    this.errorIconUrl,
    this.errorStateView,
    this.emptyStateView,
    this.emptyIconUrl,
  });

  ///[smartRepliesStyle] provides styling to the reply view
  final AiSmartReplyStyle? smartRepliesStyle;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[emptyStateText] text to be displayed when the replies are empty
  final String? emptyStateText;

  ///[onError] callback triggered in case any error happens when fetching replies
  final OnError? onError;

  ///[errorStateText] text to be displayed when error occur
  final String? errorStateText;

  ///[smartReplyView] return custom smartReply view
  final Widget Function(List<String> replies, BuildContext context)?
      smartReplyView;

  ///[onClick] function when user wants to send custom ui along with replies
  final Function(
      User? user,
      Group? group,
      Function(Widget, bool isLoading, BuildContext context) onSuccess,
      Function(CometChatException e, bool isError, BuildContext context)
          onError)? onClick;

  ///[loadingStateText] text to be displayed when loading occur
  final String? loadingStateText;

  ///[emptyStateView] returns view fow empty state
  final WidgetBuilder? emptyStateView;

  ///[loadingStateView] returns view for loading state
  final WidgetBuilder? loadingStateView;

  ///[errorStateView] returns view for error state
  final WidgetBuilder? errorStateView;

  ///[errorIconUrl] used to set the error icon
  final String? errorIconUrl;

  ///[emptyIconUrl] used to set the empty icon
  final String? emptyIconUrl;

  ///[loadingIconUrl] used to set the loading icon
  final String? loadingIconUrl;

  AiSmartReplyConfiguration fromJson(Map<String, dynamic> json) {
    return AiSmartReplyConfiguration(
      smartRepliesStyle: smartRepliesStyle ??
          json['smartRepliesStyle'] ??
          AiSmartReplyStyle().fromJson(json),
      theme: theme ?? json['theme'],
      emptyStateText: emptyStateText ?? json['emptyStateText'],
      onError: onError ?? json['onError'],
      errorStateText: errorStateText ?? json['errorStateText'],
      smartReplyView: smartReplyView ?? json['smartReplyView'],
      onClick: onClick ?? json['onClick'],
      loadingStateText: loadingStateText ?? json['loadingStateText'],
      emptyStateView: emptyStateView ?? json['emptyStateView'],
      loadingStateView: loadingStateView ?? json['loadingStateView'],
      errorStateView: errorStateView ?? json['errorStateView'],
      errorIconUrl: errorIconUrl ?? json['errorIconUrl'],
      emptyIconUrl: emptyIconUrl ?? json['emptyIconUrl'],
      loadingIconUrl: loadingIconUrl ?? json['loadingIconUrl'],
    );
  }

  /// Copies current [AiSmartReplyConfiguration] with some changes
  AiSmartReplyConfiguration copyWith({
    AiSmartReplyStyle? smartRepliesStyle,
    CometChatTheme? theme,
    String? emptyStateText,
    OnError? onError,
    String? errorStateText,
    Widget Function(List<String> replies, BuildContext context)? smartReplyView,
    Function(
            User? user,
            Group? group,
            Function(Widget, bool isLoading, BuildContext context) onSuccess,
            Function(CometChatException e, bool isError, BuildContext context)
                onError)?
        onClick,
    String? loadingStateText,
    WidgetBuilder? emptyStateView,
    WidgetBuilder? loadingStateView,
    WidgetBuilder? errorStateView,
    String? errorIconUrl,
    String? emptyIconUrl,
    String? loadingIconUrl,
  }) {
    return AiSmartReplyConfiguration(
      smartRepliesStyle: smartRepliesStyle ?? this.smartRepliesStyle,
      theme: theme ?? this.theme,
      emptyStateText: emptyStateText ?? this.emptyStateText,
      onError: onError ?? this.onError,
      errorStateText: errorStateText ?? this.errorStateText,
      smartReplyView: smartReplyView ?? this.smartReplyView,
      onClick: onClick ?? this.onClick,
      loadingStateText: loadingStateText ?? this.loadingStateText,
      emptyStateView: emptyStateView ?? this.emptyStateView,
      loadingStateView: loadingStateView ?? this.loadingStateView,
      errorStateView: errorStateView ?? this.errorStateView,
      errorIconUrl: errorIconUrl ?? this.errorIconUrl,
      emptyIconUrl: emptyIconUrl ?? this.emptyIconUrl,
      loadingIconUrl: loadingIconUrl ?? this.loadingIconUrl,
    );
  }

  /// Merges current [AiSmartReplyConfiguration] with [other]
  AiSmartReplyConfiguration merge(AiSmartReplyConfiguration? other) {
    if (other == null) return this;
    return copyWith(
      smartRepliesStyle: other.smartRepliesStyle,
      theme: other.theme,
      emptyStateText: other.emptyStateText,
      onError: other.onError ?? onError,
      errorStateText: other.errorStateText,
      smartReplyView: other.smartReplyView,
      onClick: other.onClick,
      loadingStateText: other.loadingStateText,
      emptyStateView: other.emptyStateView,
      loadingStateView: other.loadingStateView,
      errorStateView: other.errorStateView,
      errorIconUrl: other.errorIconUrl,
      emptyIconUrl: other.emptyIconUrl,
      loadingIconUrl: other.loadingIconUrl,
    );
  }
}
